//@author: vux
//@help: standard constant shader
//@tags: color
//@credits: 

////////////////////////
// User interface
////////////////////////

float normalPower <
	string UIName = "Bumpmap Strength";
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 2.0;
	float UIStep = 0.1;
> = 1.0;
bool flipGreen
<
    string UIName = "Invert Normal Map Green Channel?";
> = false;

Texture2D normalMap;
SamplerState NormalSampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};



Texture2D litSphereMap; 

SamplerState LightSampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct particles
{
	float3 pos;
	float3 vel;
	float3 col;
	float size;
};

StructuredBuffer<particles> pData;



cbuffer cbPerDraw : register( b0 )
{
	float4x4 tW : WORLD;
	float4x4 tWV : WORLDVIEW;
	float4x4 tVP : VIEWPROJECTION;
	int colorcount = 1;
};

float4x4 tWVP: WORLDVIEWPROJECTION;
struct VS_IN
{
	uint ii : SV_InstanceID;
	float4 PosO : POSITION;
	float2 TexCoord : TEXCOORD0;
//	float4 Tangent  : TANGENT;
//	float4 Binormal : BINORMAL;
	float4 Normal   : NORMAL;

};

struct vs2ps
{
    float4 PosWVP: SV_POSITION;	
	float4 Color: COLOR0;
	float2 TexCoord : TEXCOORD1;
	float4 TexCoord1 : TEXCOORD2;
	float4 TexCoord2 : TEXCOORD3;
	float4 TexCoord3 : TEXCOORD4;
	float4 Vcol : COLOR2 ;
	
};

vs2ps VS(VS_IN input)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
	float4 p = float4(pData[input.ii].pos,0); //particle Pos
	float3 c = pData[input.ii].col; //particle colour
	float4 pv = mul(input.PosO,tW); //Transform the model
	float4 PosV = pv + p;
//	float4 vtan =  input.Tangent;
//	float4 vbinorm = -input.Binormal;
	float4 vnorm = input.Normal;

//	Out.TexCoord1 = mul(vtan,tWV);
//	Out.TexCoord2 = mul(vbinorm,tWV);
	Out.TexCoord3 = mul(vnorm,tWV);	
	
	
	Out.PosWVP =  mul(PosV, tVP);	

    Out.TexCoord = input.TexCoord;
	Out.Vcol = float4(c,1);
    return Out;
}




float4 PS_Tex(vs2ps In): SV_Target
{
    float3 texNormal = normalMap.Sample( NormalSampler, In.TexCoord).xyz ;
		if(flipGreen)texNormal.g = 1-texNormal.g;
	texNormal.rgb = texNormal.rgb*2.0-1.0;
	// Fixes normals if no normal map texture is supplied
	if ( dot(texNormal,texNormal) > 2.0 ) {
		texNormal = float3(0,0,1);
	}
	
	// The normalizes can probably go away... but meh...
    float3 T = normalize(In.TexCoord1.xyz) * normalPower;
    float3 B = normalize(In.TexCoord2.xyz) * normalPower;
    float3 N = normalize(In.TexCoord3.xyz);
	
	// Put in world space and renormalize (after scaling)
    float3 worldNorm = normalize(N + texNormal.y * B + texNormal.x * T);

	// Swap it around a bit... 
	worldNorm.y = - worldNorm.y;

	// Look up in the litsphere!
	float3 light = litSphereMap.Sample( LightSampler, worldNorm.xy * 0.5 + 0.5).xyz ;
  
	return In.Vcol;
}





technique10 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS_Tex() ) );
	}
}




